@@ -12,20 +12,16 @@
 
 from owslib.etree import etree
 from owslib import util
+from owslib.namespaces import Namespaces
 
 # default variables
+def get_namespaces():
+    n = Namespaces()
+    ns = n.get_namespaces(["gco","gmd","gml","gml32","gmx","gts","srv","xlink"])
+    ns[None] = n.get_namespace("gmd")
+    return ns
+namespaces = get_namespaces()
 
-namespaces = {
-    None : 'http://www.isotc211.org/2005/gmd',
-    'gco': 'http://www.isotc211.org/2005/gco',
-    'gmd': 'http://www.isotc211.org/2005/gmd',
-    'gml': 'http://www.opengis.net/gml',
-    'gml32': 'http://www.opengis.net/gml/3.2',
-    'gmx': 'http://www.isotc211.org/2005/gmx',
-    'gts': 'http://www.isotc211.org/2005/gts',
-    'srv': 'http://www.isotc211.org/2005/srv',
-    'xlink': 'http://www.w3.org/1999/xlink'
-}
 
 class MD_Metadata(object):
     """ Process gmd:MD_Metadata """
@@ -332,11 +328,9 @@ class MD_DataIdentification(object):
         # from the one containing either an EX_GeographicBoundingBox or EX_BoundingPolygon.
         # The schema also specifies an EX_GeographicDescription. This is not implemented yet.
         val = None
-        extent = md.find(util.nspath_eval('gmd:extent', namespaces))
-        if extent is None:
-            extent = md.find(util.nspath_eval('srv:extent', namespaces))
-        
-        if extent is not None:
+        extents = md.findall(util.nspath_eval('gmd:extent', namespaces))
+        extents.extend(md.findall(util.nspath_eval('srv:extent', namespaces)))
+        for extent in extents:
             for e in extent.findall(util.nspath_eval('gmd:EX_Extent/gmd:geographicElement', namespaces)):
                 if e.find(util.nspath_eval('gmd:EX_GeographicBoundingBox', namespaces)) is not None or e.find(util.nspath_eval('gmd:EX_BoundingPolygon', namespaces)) is not None:
                     val = e
